package com.frist008.nixlibrary;

import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v7.widget.RecyclerView;

import com.frist008.nixlibrary.utils.NixWrapperUri;

/**
 * It simple builder for register {@link android.support.v4.content.CursorLoader}. For registration
 * of the loader use the static method {@link #create} and after then use a {@link #register}. You
 * can used other methods for configure his a query to database
 */
@SuppressWarnings("unused")
public class LoaderRecyclerBuilder {

	private final NixLoaderBuilder builder;

	LoaderRecyclerBuilder(@NonNull FragmentActivity activity,
	                      @Nullable Fragment fragment,
	                      @NonNull Uri uri) {
		builder = new NixLoaderBuilder(activity, fragment, uri);
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param recyclerView For of adapter this view will called changeCursor
	 * @param loaderId     The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public CursorRecyclerLoader register(@NonNull RecyclerView recyclerView, int loaderId) {
		return new CursorRecyclerLoader(builder.fragment,
		                                builder.activity,
		                                recyclerView,
		                                loaderId,
		                                builder.listenerBefore,
		                                builder.listenerAfter,
		                                builder.whereListener,
		                                builder.uri,
		                                builder.projection,
		                                builder.sort,
		                                builder.select,
		                                builder.args);
	}

	/**
	 * {@link NixLoaderBuilder#create(NixWrapperUri, Fragment)}
	 */
	public static LoaderRecyclerBuilder create(@NonNull NixWrapperUri nixUri,
	                                           @NonNull Fragment fragment) {
		return create(nixUri.getUri(), fragment);
	}

	/**
	 * {@link NixLoaderBuilder#create(Uri, Fragment)}
	 */
	public static LoaderRecyclerBuilder create(@NonNull Uri uri, @NonNull Fragment fragment) {
		return new LoaderRecyclerBuilder(fragment.getActivity(), fragment, uri);
	}

	/**
	 * {@link NixLoaderBuilder#create(NixWrapperUri, FragmentActivity)}
	 */
	public static LoaderRecyclerBuilder create(@NonNull NixWrapperUri nixUri,
	                                           @NonNull FragmentActivity activity) {
		return create(nixUri.getUri(), activity);
	}

	/**
	 * {@link NixLoaderBuilder#create(Uri, FragmentActivity)}
	 */
	public static LoaderRecyclerBuilder create(@NonNull Uri uri,
	                                           @NonNull FragmentActivity activity) {
		return new LoaderRecyclerBuilder(activity, null, uri);
	}

	/**
	 * {@link NixLoaderBuilder#projection(String...)}
	 */
	public LoaderRecyclerBuilder projection(@Nullable String... projection) {
		builder.projection(projection);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#sortAsc(String, String...)}
	 */
	public LoaderRecyclerBuilder sortAsc(String field, @Nullable String... fields) {
		builder.sortAsc(field, fields);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#sortDesc(String, String...)}
	 */
	public LoaderRecyclerBuilder sortDesc(String field, @Nullable String... fields) {
		builder.sortDesc(field, fields);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#where(String, String...)}
	 */
	public LoaderRecyclerBuilder where(@NonNull String select, @NonNull String... args) {
		builder.where(select, args);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#where(String)}
	 */
	public LoaderRecyclerBuilder where(@NonNull String select) {
		builder.where(select);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#setWhereListener(OnChangeWhereListener)}
	 */
	public LoaderRecyclerBuilder setWhereListener(
			@NonNull OnChangeWhereListener selectListener) {
		builder.setWhereListener(selectListener);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#changeCursorBeforeListener(OnChangeCursorBeforeListener)}
	 */
	public LoaderRecyclerBuilder changeCursorBeforeListener(
			@NonNull OnChangeCursorBeforeListener changeCursorListener) {
		builder.changeCursorBeforeListener(changeCursorListener);
		return this;
	}

	/**
	 * {@link NixLoaderBuilder#changeCursorAfterListener(OnChangeCursorAfterListener)}
	 */
	public LoaderRecyclerBuilder changeCursorAfterListener(
			@NonNull OnChangeCursorAfterListener changeCursorListener) {
		builder.changeCursorAfterListener(changeCursorListener);
		return this;
	}
}
